#!/usr/bin/python3
# -*- encoding: UTF-8 -*-

# Imports
try:
    import yaml, psycopg2, logging, os, sys, subprocess, shutil, platform
    from pathlib import Path
    from datetime import date
    from psycopg2 import sql
    from psycopg2.extensions import AsIs
except Exception as e:
    print(e)
    sys.exit(1)

# Paramètres config.yaml
try:
    with open("../config.yaml", 'rt', encoding='UTF-8') as yml:
        config = yaml.safe_load(yml)
        lbtHost = config['database']['host']
        lbtPort = config['database']['port']
        lbtDb = config['database']['dbname']
        lbtUser = config['database']['username']
        lbtPassword = config['database']['password']
        lbtSchema = config['database']['schema']
        lbtSrid = int(config['database']['srid'])
except Exception as e:
    print(e)
    sys.exit(1)

# Variable(s)
vToday = date.today()

# Fichier journal (DEBUG/INFO/WARNING/ERROR/CRITICAL)
try:
    logDirectory = '../log/lbt-load-data/'
    if not os.path.exists(logDirectory):
        os.makedirs(logDirectory)
    logFilename = logDirectory + str(vToday) + '.log'
    logging.basicConfig(filename=logFilename,filemode='w',format='%(asctime)s|%(levelname)s|%(message)s',datefmt='%d/%m/%Y %H:%M:%S',level=logging.INFO)
    logging.info('Start Python script.')
except Exception as e:
    print(e)
    sys.exit(1)

# Environnement
platform = platform.system()
logging.info('OS: ' + platform)
current_directory = os.getcwd().replace(os.sep, '/')
logging.info('Current directory: ' + current_directory)
work_directory = str(Path(current_directory).parent.absolute()).replace(os.sep, '/')
logging.info('Work directory: ' + work_directory)

# Constantes
osm_ref_data_file = work_directory + '/osm/source/lbt-osm-ref-data.osm'
osm_pgr_data_file = work_directory + '/osm/source/lbt-osm-pgr-data.osm'

# Ouvre la connexion PG sur LeBonTag
try:
    logging.info('Open database connection.')
    connect_str = "host=%s port=%s dbname=%s user=%s password=%s" % (lbtHost,lbtPort,lbtDb,lbtUser,lbtPassword)
    conn = psycopg2.connect(connect_str)
    cursor = conn.cursor()
except Exception as e:
    logging.critical('PG connection error : ' + str(e))
    sys.exit(1)

# Récupère les paramètres stockés en base de données
try:
    cursor.execute(sql.SQL("SELECT s_name,s_value FROM {0}.lbt_setting;").format(sql.Identifier(lbtSchema)))
    rows = cursor.fetchall()
    drows = dict(rows)
    db_ref_srid = int(drows['db_ref_srid'])
    db_ref_schema = drows['db_ref_schema']
    db_ref_prefix = drows['db_ref_prefix']
    db_pgr_schema = drows['db_pgr_schema']
    db_pgr_prefix = drows['db_pgr_prefix']
    osm_mapconfigfile = drows['osm_mapconfigfile']
    osm_pbffile = drows['osm_pbffile']
    osm_polyfile = drows['osm_polyfile']
    osm_stylefile = drows['osm_stylefile']

except Exception as e:
    logging.critical('LeBonTag settings fetching error : ' + str(e))
    sys.exit(1)

# Découpe le fichier PBF des données de référence et le convertit en OSM
try:
    logging.info('Split/copy PBF file...')
    if osm_pbffile != None:
        if ((osm_polyfile != None) and (osm_polyfile != '')):
            #proc = subprocess.run(['osmconvert',osm_pbffile,'-B=' + osm_polyfile,'--complete-ways','--complex-ways','--out-osm','-o=' + osm_ref_data_file])
            #with open(osm_ref_data_file,'w') as fout:
                #proc = subprocess.Popen(['osmconvert',osm_pbffile,'-B=' + osm_polyfile,'--complete-ways','--complex-ways','--out-osm'],stdout=fout)
            if platform == 'Windows':
                os.system(work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe ' + osm_pbffile + ' -B=' + osm_polyfile + ' --complete-ways --complex-ways --out-osm -o=' + osm_ref_data_file)
            else:
                os.system('osmconvert ' + osm_pbffile + ' -B=' + osm_polyfile + ' --complete-ways --complex-ways --out-osm -o=' + osm_ref_data_file)
        else:
            #proc = subprocess.run(['osmconvert',osm_pbffile,'--complete-ways','--complex-ways','--out-osm','-o=' + osm_ref_data_file])
            #with open(osm_ref_data_file,'w') as fout:
                #proc = subprocess.Popen(['osmconvert',osm_pbffile,'--complete-ways','--complex-ways','--out-osm'],stdout=fout)
            if platform == 'Windows':
                os.system(work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe ' + osm_pbffile + ' --complete-ways --complex-ways --out-osm -o=' + osm_ref_data_file)
            else:
                os.system('osmconvert ' + osm_pbffile + ' --complete-ways --complex-ways --out-osm -o=' + osm_ref_data_file)
except Exception as e:
    logging.critical('PBF file splitting or conversion error : ' + str(e))
    sys.exit(1)

# Charge les données de référence OSM, avec ou sans fichier de style spécifique ("default.style" sera utilisé par défaut par le programme)
# Sous Windows, force le chargement en 4326 (sinon erreur de dépendance avec proj4)
try:

    logging.info('Load OSM file with osm2pgsql...')

    logging.info('Change search_path for osm2pgsql.')
    cursor.execute(sql.SQL("ALTER ROLE {0} SET search_path TO {1}, public;").format(sql.Identifier(lbtUser),sql.Identifier(db_ref_schema)))
    conn.commit()

    if os.path.isfile(osm_ref_data_file):
        if osm_stylefile != None:
            if platform == 'Windows':
                proc = subprocess.run([work_directory + '/tools/osm2pgsql/osm2pgsql.exe','--create','--slim','--proj','4326','--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--style',osm_stylefile,'--input-reader','osm',osm_ref_data_file],env=dict(os.environ,PGPASSWORD=lbtPassword))
            else:
                proc = subprocess.run(['osm2pgsql','--create','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--style',osm_stylefile,'--input-reader','osm',osm_ref_data_file],env=dict(os.environ,PGPASSWORD=lbtPassword))
        else:
            if platform == 'Windows':
                proc = subprocess.run([work_directory + '/tools/osm2pgsql/osm2pgsql.exe','--create','--slim','--proj','4326','--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--input-reader','osm',osm_ref_data_file],env=dict(os.environ,PGPASSWORD=lbtPassword))
            else:
                proc = subprocess.run(['osm2pgsql','--create','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--input-reader','osm',osm_ref_data_file],env=dict(os.environ,PGPASSWORD=lbtPassword))

    logging.info('Change search_path for osm2pgsql.')
    cursor.execute(sql.SQL("ALTER ROLE {0} SET search_path TO public, {1};").format(sql.Identifier(lbtUser),sql.Identifier(db_ref_schema)))
    conn.commit()

    # PATCH : Fusion des grandes ways découpées par osm2pgsql
    logging.info('SQL Patch for osm2pgsql...')
    cursor.execute(sql.SQL("WITH duplicates AS ( DELETE FROM {0}.{1} WHERE osm_id IN (SELECT osm_id FROM {0}.{1} GROUP BY osm_id, name, tags HAVING COUNT(osm_id) > 1 AND osm_id > 0) RETURNING * ) INSERT INTO {0}.{1} (osm_id, name, tags, way) SELECT osm_id, name, tags, ST_LineMerge(ST_Union(way)) as way FROM duplicates GROUP BY osm_id, name, tags;").format(sql.Identifier(db_ref_schema),sql.Identifier(db_ref_prefix + '_line')))

except Exception as e:
    logging.critical('OSM file loading error : ' + str(e))
    sys.exit(1)

# Nettoie le fichier pour pgRouting
try:
    logging.info('Clean OSM file for pgRouting...')
    #with open(osm_pgr_data_file,'w') as fout:
        #proc = subprocess.Popen(['osmfilter',osm_ref_data_file,'--drop-version','--keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian"','--out-osm'],stdout=fout)
    #proc = subprocess.run(['osmfilter',osm_ref_data_file,'--drop-version','--keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian"','--out-osm','-o=' + osm_pgr_data_file])
    if platform == 'Windows':
        os.system(work_directory + '/tools/osmfilter/osmfilter.exe ' + osm_ref_data_file + ' --drop-version --keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian" --out-osm -o=' + osm_pgr_data_file)
    else:
        os.system('osmfilter ' + osm_ref_data_file + ' --drop-version --keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian" --out-osm -o=' + osm_pgr_data_file)

except Exception as e:
    logging.critical('osmfilter cleaning error : ' + str(e))
    sys.exit(1)

# Charge le fichier via pgRouting
try:
    logging.info('Load OSM file with pgRouting...')
    if platform == 'Windows':
        proc = subprocess.run([work_directory + '/tools/osm2pgrouting/osm2pgrouting.exe','--file',osm_pgr_data_file,'--conf',osm_mapconfigfile,'--host',lbtHost,'--port',str(lbtPort),'--dbname',lbtDb,'--username',lbtUser,'--password',lbtPassword,'--schema',db_pgr_schema,'--prefix',db_pgr_prefix + '_','--clean'])
    else:
        proc = subprocess.run(['osm2pgrouting','--file',osm_pgr_data_file,'--conf',osm_mapconfigfile,'--host',lbtHost,'--port',str(lbtPort),'--dbname',lbtDb,'--username',lbtUser,'--password',lbtPassword,'--schema',db_pgr_schema,'--prefix',db_pgr_prefix + '_','--clean'])
except Exception as e:
    logging.critical('pgRouting loading error : ' + str(e))
    sys.exit(1)

# Ferme la connexion PG
try:
    logging.info('Close database connection.')
    cursor.close()
    conn.close()
except Exception as e:
    logging.critical('Error when closing PG : ' + str(e))
    sys.exit(1)
