#!/usr/bin/python3
# -*- encoding: UTF-8 -*-

# Imports
try:
    import yaml, os, shutil, psycopg2, gzip, time, logging, sys, subprocess, platform
    from pathlib import Path
    from datetime import date, timedelta, datetime
    from psycopg2 import sql
    from psycopg2.extensions import AsIs
    from lxml import etree as ET
except Exception as e:
    print(e)
    sys.exit(1)

# Paramètres config.yaml
try:
    with open("../config.yaml", 'rt', encoding='UTF-8') as yml:
        config = yaml.safe_load(yml)
        lbtHost = config['database']['host']
        lbtPort = config['database']['port']
        lbtDb = config['database']['dbname']
        lbtUser = config['database']['username']
        lbtPassword = config['database']['password']
        lbtSchema = config['database']['schema']
        lbtSrid = int(config['database']['srid'])
except Exception as e:
    print(e)
    sys.exit(1)

# Variable(s) & constante(s)
vToday = date.today()
vYesterday = vToday + timedelta(days=-1)

# Fichier journal (DEBUG/INFO/WARNING/ERROR/CRITICAL)
try:
    logDirectory = '../log/lbt-update-db-by-osc/' + str(vToday.year) + str('%02d' % vToday.month) + '/'
    if not os.path.exists(logDirectory):
        os.makedirs(logDirectory)
    logFilename = logDirectory + str(vToday) + '.log'
    logging.basicConfig(filename=logFilename,filemode='w',format='%(asctime)s|%(levelname)s|%(message)s',datefmt='%d/%m/%Y %H:%M:%S',level=logging.INFO)
    logging.info('Start Python script.')
except Exception as e:
    print(e)
    sys.exit(1)

# Environnement
platform = platform.system()
logging.info('OS: ' + platform)
current_directory = os.getcwd().replace(os.sep, '/')
logging.info('Current directory: ' + current_directory)
work_directory = str(Path(current_directory).parent.absolute()).replace(os.sep, '/')
logging.info('Work directory: ' + work_directory)

# Constantes
osm_ref_data_file = work_directory + '/osm/source/lbt-osm-ref-data.osm'
osm_ref_data_file_updated = work_directory + '/osm/source/lbt-osm-ref-data-updated.osm'
osm_pgr_data_updated_file = work_directory + '/osm/source/lbt-osm-pgr-updated-data.osm'
#osm_pgr_data_file = '/opt/le-bon-tag/osm/source/lbt-osm-pgr-data.osm'
#osm_ref_data_file_updated_tmp = '/opt/le-bon-tag/osm/source/lbt-osm-ref-data-updated-tmp.osm'

# Ouvre la connexion PG sur LeBonTag
try:
    logging.info('Open database connection.')
    connect_str = "host=%s port=%s dbname=%s user=%s password=%s" % (lbtHost,lbtPort,lbtDb,lbtUser,lbtPassword)
    conn = psycopg2.connect(connect_str)
    cursor = conn.cursor()
except Exception as e:
    logging.critical('PG connection error : ' + str(e))
    sys.exit(1)

# Récupère les paramètres stockés en base de données
try:
    cursor.execute(sql.SQL("SELECT s_name,s_value FROM {0}.lbt_setting;").format(sql.Identifier(lbtSchema)))
    rows = cursor.fetchall()
    drows = dict(rows)
    db_ref_update = drows['db_ref_update']
    db_ref_srid = int(drows['db_ref_srid'])
    db_ref_schema = drows['db_ref_schema']
    db_ref_prefix = drows['db_ref_prefix']
    db_pgr_update = drows['db_pgr_update']
    db_pgr_schema = drows['db_pgr_schema']
    db_pgr_prefix = drows['db_pgr_prefix']
    osm_mapconfigfile = drows['osm_mapconfigfile']
    osm_pbffile = drows['osm_pbffile']
    osm_polyfile = drows['osm_polyfile']
    osm_stylefile = drows['osm_stylefile']
except Exception as e:
    logging.critical('LeBonTag settings fetching error : ' + str(e))
    sys.exit(1)

# Faut-il mettre à jour les bases ?
try:
    if db_ref_update == '0' and db_pgr_update == '0':
        logging.info('No database to update. Exiting.')
        sys.exit(0)
except Exception as e:
    logging.critical('Settings error : ' + str(e))
    sys.exit(1)

# Récupère les OSC de tous les objets validés à intégrer en base et crée l'OSC final de mise à jour via osmconvert (supprime les doublons et ordonne correctement les objets) pour la base osm2pgsql
try:
    logging.info('Retrieve all OSC fragments from validated objects.')

    cursor.execute(sql.SQL("SELECT od_id,od_osc,od_new_timestamp,od_status,od_id_nodes,od_id_members FROM {0}.lbt_osmdiff WHERE od_osc IS NOT NULL AND od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'valid');").format(sql.Identifier(lbtSchema)))
    rows = cursor.fetchall()

    if cursor.rowcount > 0:

        # Initialisation du fichier final (vide)
        xmlDocumentDirectory = '../osm/osc_update/' + str(vToday.year) + str('%02d' % vToday.month) + '/'
        if not os.path.exists(xmlDocumentDirectory):
            os.makedirs(xmlDocumentDirectory)
        xmlFile = xmlDocumentDirectory + str(vYesterday) + '.osc'
        top = ET.Element('osmChange')
        top.set('version', '0.6')
        top.set('generator','LeBonTag')
        xmlDocument = ET.ElementTree(top)
        xmlDocument.write(xmlFile, encoding='UTF-8', xml_declaration=True, method='xml')

        # Écrit le fragment XML (osc) de l'objet validé dans un fichier temporaire et le fusionne avec le fichier final via osmconvert
        for row in rows:
            logging.info('Write OSC fragment for object ' + str(row[0]))
            od_id = row[0]
            xmlFragment = row[1].encode('UTF-8')
            od_new_timestamp = row[2]
            od_status = row[3]
            od_id_nodes = row[4]
            od_id_members = row[5]
            # Fichier temporaire du fragment à traiter
            xmlFragmentFile = open('../tmp/fragment.osc','wb')
            xmlFragmentFile.write(xmlFragment)
            xmlFragmentFile.close()
            # Fusion du fichier final avec le fragment dans un fichier temporaire
            with open(xmlFile + '.tmp','w') as fout:
                if platform == 'Windows':
                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                else:
                    proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                return_code = proc.wait()
            # Le fichier temporaire devient le fichier final
            os.remove(xmlFile)
            os.remove('../tmp/fragment.osc')
            shutil.move(xmlFile + '.tmp',xmlFile)

            # Récupère les nodes associés à l'objet validé (si c'est une way)
            # Correction #58 : il faut aussi générer les OSC des nodes hors emprise de la way car ils n'ont pas été chargés dans osmdiff
            # Donc on doit tester les nodes 1 par 1.
            if od_id_nodes is not None and len(od_id_nodes):
                root = ET.fromstring(xmlFragment)
                for node in od_id_nodes:

                    # Recherche le node dans lbt_osmdiff
                    cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff_history WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(node,od_new_timestamp,node,od_new_timestamp,))
                    nodesRows = cursor.fetchall()

                    # Si le node est trouvé au moins une fois dans lbt_osmdiff, on inclut son propre OSC
                    if nodesRows is not None and len(nodesRows):
                        for row in nodesRows:
                            logging.info('Write OSC fragment for node ' + str(row[0]))
                            od_id = row[0]
                            od_new_timestamp = row[1]
                            xmlFragment = row[2].encode('UTF-8')
                            # Fichier temporaire du fragment à traiter
                            xmlFragmentFile = open('../tmp/fragment.osc','wb')
                            xmlFragmentFile.write(xmlFragment)
                            xmlFragmentFile.close()
                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                            with open(xmlFile + '.tmp','w') as fout:
                                if platform == 'Windows':
                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                else:
                                    proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                return_code = proc.wait()
                            # Le fichier temporaire devient le fichier final
                            os.remove(xmlFile)
                            os.remove('../tmp/fragment.osc')
                            shutil.move(xmlFile + '.tmp',xmlFile)

                    # Si le node n'est pas trouvé dans lbt_osmdiff (car hors emprise), on génère son propre OSC
                    elif nodesRows is not None and not len(nodesRows):
                        logging.info('Node outside perimeter ' + str(node))
                        # Récupère les coordonnées du node depuis le OSC de la way
                        nodeLat = root.xpath("//nd[@ref='%s']/@lat" % node)[0]
                        nodeLon = root.xpath("//nd[@ref='%s']/@lon" % node)[0]

                        # Base de référence (table point)
                        cursor.execute(sql.SQL("SELECT osm_id FROM {0}.{1} WHERE osm_id = %s;").format(sql.Identifier(db_ref_schema),sql.Identifier(db_ref_prefix + '_point')),(node,))
                        row = cursor.fetchone()

                        # Si le node est trouvé dans la base de référence, on crée un OSC de type modify
                        if row is not None and nodeLon is not None and nodeLat is not None:
                            top = ET.Element('osmChange')
                            top.set('version', '0.6')
                            top.set('generator','LeBonTag')
                            topChild = ET.SubElement(top, 'modify')
                            topChildElement = ET.SubElement(topChild, 'node')
                            topChildElement.set('id',str(node))
                            topChildElement.set('lat',str(nodeLat))
                            topChildElement.set('lon',str(nodeLon))
                            oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                            # Fichier temporaire du fragment à traiter
                            xmlFragmentFile = open('../tmp/fragment.osc','w')
                            xmlFragmentFile.write(oscContent)
                            xmlFragmentFile.close()
                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                            with open(xmlFile + '.tmp','w') as fout:
                                if platform == 'Windows':
                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                else:
                                    proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                return_code = proc.wait()
                            # Le fichier temporaire devient le fichier final
                            os.remove(xmlFile)
                            os.remove('../tmp/fragment.osc')
                            shutil.move(xmlFile + '.tmp',xmlFile)

                        # Si le node n'est pas trouvé dans la base de référence, on crée un OSC de type create
                        elif row is None and nodeLon is not None and nodeLat is not None:
                            top = ET.Element('osmChange')
                            top.set('version', '0.6')
                            top.set('generator','LeBonTag')
                            topChild = ET.SubElement(top, 'create')
                            topChildElement = ET.SubElement(topChild, 'node')
                            topChildElement.set('id',str(node))
                            topChildElement.set('lat',str(nodeLat))
                            topChildElement.set('lon',str(nodeLon))
                            oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                            # Fichier temporaire du fragment à traiter
                            xmlFragmentFile = open('../tmp/fragment.osc','w')
                            xmlFragmentFile.write(oscContent)
                            xmlFragmentFile.close()
                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                            with open(xmlFile + '.tmp','w') as fout:
                                if platform == 'Windows':
                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                else:
                                    proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                return_code = proc.wait()
                            # Le fichier temporaire devient le fichier final
                            os.remove(xmlFile)
                            os.remove('../tmp/fragment.osc')
                            shutil.move(xmlFile + '.tmp',xmlFile)

            # Pour les relations, traite chacun des membres qui peuvent être des nodes ou des ways
            if od_id_members is not None and len(od_id_members):
                cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc,od_id_nodes FROM {0}.lbt_osmdiff WHERE od_id = ANY (%s) AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc,od_id_nodes FROM {0}.lbt_osmdiff_history WHERE od_id = ANY (%s) AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(od_id_members,od_new_timestamp,od_id_members,od_new_timestamp,))
                membersRows = cursor.fetchall()
                if membersRows is not None:
                    for membersRow in membersRows:
                        od_id = membersRow[0]
                        od_new_timestamp = membersRow[1]
                        xmlFragment = membersRow[2].encode('UTF-8')
                        od_id_nodes = membersRow[3]

                        # On écrit le fragment OSC de chaque membre (node ou way)
                        # Fichier temporaire du fragment à traiter
                        xmlFragmentFile = open('../tmp/fragment.osc','wb')
                        xmlFragmentFile.write(xmlFragment)
                        xmlFragmentFile.close()
                        # Fusion du fichier final avec le fragment dans un fichier temporaire
                        with open(xmlFile + '.tmp','w') as fout:
                            if platform == 'Windows':
                                proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                            else:
                                proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                            return_code = proc.wait()
                        # Le fichier temporaire devient le fichier final
                        os.remove(xmlFile)
                        os.remove('../tmp/fragment.osc')
                        shutil.move(xmlFile + '.tmp',xmlFile)

                        # Récupère les nodes associés à l'objet validé (si c'est une way)
                        # Correction #58 : il faut aussi générer les OSC des nodes hors emprise de la way car ils n'ont pas été chargés dans osmdiff
                        # Donc on doit tester les nodes 1 par 1.
                        if od_id_nodes is not None and len(od_id_nodes):
                            root = ET.fromstring(xmlFragment)
                            for node in od_id_nodes:

                                # Recherche le node dans lbt_osmdiff
                                cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff_history WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(node,od_new_timestamp,node,od_new_timestamp,))
                                nodesRows = cursor.fetchall()

                                # Si le node est trouvé au moins une fois dans lbt_osmdiff, on inclut son propre OSC
                                if nodesRows is not None and len(nodesRows):
                                    for row in nodesRows:
                                        logging.info('Write OSC fragment for node ' + str(row[0]))
                                        od_id = row[0]
                                        od_new_timestamp = row[1]
                                        xmlFragment = row[2].encode('UTF-8')
                                        # Fichier temporaire du fragment à traiter
                                        xmlFragmentFile = open('../tmp/fragment.osc','wb')
                                        xmlFragmentFile.write(xmlFragment)
                                        xmlFragmentFile.close()
                                        # Fusion du fichier final avec le fragment dans un fichier temporaire
                                        with open(xmlFile + '.tmp','w') as fout:
                                            if platform == 'Windows':
                                                proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            else:
                                                proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            return_code = proc.wait()
                                        # Le fichier temporaire devient le fichier final
                                        os.remove(xmlFile)
                                        os.remove('../tmp/fragment.osc')
                                        shutil.move(xmlFile + '.tmp',xmlFile)

                                # Si le node n'est pas trouvé dans lbt_osmdiff (car hors emprise), on génère son propre OSC
                                elif nodesRows is not None and not len(nodesRows):
                                    logging.info('Node outside perimeter ' + str(node))
                                    # Récupère les coordonnées du node depuis le OSC de la way
                                    nodeLat = root.xpath("//nd[@ref='%s']/@lat" % node)[0]
                                    nodeLon = root.xpath("//nd[@ref='%s']/@lon" % node)[0]

                                    # Base de référence (table point)
                                    cursor.execute(sql.SQL("SELECT osm_id FROM {0}.{1} WHERE osm_id = %s;").format(sql.Identifier(db_ref_schema),sql.Identifier(db_ref_prefix + '_point')),(node,))
                                    row = cursor.fetchone()

                                    # Si le node est trouvé dans la base de référence, on crée un OSC de type modify
                                    if row is not None and nodeLon is not None and nodeLat is not None:
                                        top = ET.Element('osmChange')
                                        top.set('version', '0.6')
                                        top.set('generator','LeBonTag')
                                        topChild = ET.SubElement(top, 'modify')
                                        topChildElement = ET.SubElement(topChild, 'node')
                                        topChildElement.set('id',str(node))
                                        topChildElement.set('lat',str(nodeLat))
                                        topChildElement.set('lon',str(nodeLon))
                                        oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                        # Fichier temporaire du fragment à traiter
                                        xmlFragmentFile = open('../tmp/fragment.osc','w')
                                        xmlFragmentFile.write(oscContent)
                                        xmlFragmentFile.close()
                                        # Fusion du fichier final avec le fragment dans un fichier temporaire
                                        with open(xmlFile + '.tmp','w') as fout:
                                            if platform == 'Windows':
                                                proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            else:
                                                proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            return_code = proc.wait()
                                        # Le fichier temporaire devient le fichier final
                                        os.remove(xmlFile)
                                        os.remove('../tmp/fragment.osc')
                                        shutil.move(xmlFile + '.tmp',xmlFile)

                                    # Si le node n'est pas trouvé dans la base de référence, on crée un OSC de type create
                                    elif row is None and nodeLon is not None and nodeLat is not None:
                                        top = ET.Element('osmChange')
                                        top.set('version', '0.6')
                                        top.set('generator','LeBonTag')
                                        topChild = ET.SubElement(top, 'create')
                                        topChildElement = ET.SubElement(topChild, 'node')
                                        topChildElement.set('id',str(node))
                                        topChildElement.set('lat',str(nodeLat))
                                        topChildElement.set('lon',str(nodeLon))
                                        oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                        # Fichier temporaire du fragment à traiter
                                        xmlFragmentFile = open('../tmp/fragment.osc','w')
                                        xmlFragmentFile.write(oscContent)
                                        xmlFragmentFile.close()
                                        # Fusion du fichier final avec le fragment dans un fichier temporaire
                                        with open(xmlFile + '.tmp','w') as fout:
                                            if platform == 'Windows':
                                                proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            else:
                                                proc = subprocess.Popen(['osmconvert',xmlFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                            return_code = proc.wait()
                                        # Le fichier temporaire devient le fichier final
                                        os.remove(xmlFile)
                                        os.remove('../tmp/fragment.osc')
                                        shutil.move(xmlFile + '.tmp',xmlFile)

    else:
        logging.info('No validated objects to update. Exiting.')
        sys.exit(0)

except Exception as e:
    logging.critical('OSC fetching error : ' + str(e))
    sys.exit(1)

# Si le fichier full.osc n'existe pas (première exécution), on le génère à partir de tous les objets intégrés et validés présents en base
# Sinon on y ajoute les objets validés précédents (xmlFile)
try:

    xmlDocumentDirectory = '../osm/osc_update/'
    xmlFullFile = xmlDocumentDirectory + 'full.osc'

    if not os.path.isfile(xmlFullFile):

        logging.info('Retrieve all OSC fragments from integrated and valided objects.')

        cursor.execute(sql.SQL("SELECT od_id,od_osc,od_new_timestamp,od_status,od_id_nodes,od_id_members FROM {0}.lbt_osmdiff WHERE od_osc IS NOT NULL AND (od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'valid') OR od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated'));").format(sql.Identifier(lbtSchema)))
        rows = cursor.fetchall()

        if cursor.rowcount > 0:

            # Initialisation du fichier final (vide)

            top = ET.Element('osmChange')
            top.set('version', '0.6')
            top.set('generator','LeBonTag')
            xmlDocument = ET.ElementTree(top)
            xmlDocument.write(xmlFullFile, encoding='UTF-8', xml_declaration=True, method='xml')

            # Écrit le fragment XML (osc) de l'objet validé dans un fichier temporaire et le fusionne avec le fichier final via osmconvert
            for row in rows:
                logging.info('Write OSC fragment for object ' + str(row[0]))
                od_id = row[0]
                xmlFragment = row[1].encode('UTF-8')
                od_new_timestamp = row[2]
                od_status = row[3]
                od_id_nodes = row[4]
                od_id_members = row[5]
                # Fichier temporaire du fragment à traiter
                xmlFragmentFile = open('../tmp/fragment.osc','wb')
                xmlFragmentFile.write(xmlFragment)
                xmlFragmentFile.close()
                # Fusion du fichier final avec le fragment dans un fichier temporaire
                with open(xmlFullFile + '.tmp','w') as fout:
                    if platform == 'Windows':
                        proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                    else:
                        proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                    return_code = proc.wait()
                # Le fichier temporaire devient le fichier final
                os.remove(xmlFullFile)
                os.remove('../tmp/fragment.osc')
                shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                # Récupère les nodes associés à l'objet validé (si c'est une way)
                # Correction #58 : il faut aussi générer les OSC des nodes hors emprise de la way car ils n'ont pas été chargés dans osmdiff
                # Donc on doit tester les nodes 1 par 1.
                if od_id_nodes is not None and len(od_id_nodes):
                    root = ET.fromstring(xmlFragment)
                    for node in od_id_nodes:

                        # Recherche le node dans lbt_osmdiff
                        cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff_history WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(node,od_new_timestamp,node,od_new_timestamp,))
                        nodesRows = cursor.fetchall()

                        # Si le node est trouvé au moins une fois dans lbt_osmdiff, on inclut son propre OSC
                        if nodesRows is not None and len(nodesRows):
                            for row in nodesRows:
                                logging.info('Write OSC fragment for node ' + str(row[0]))
                                od_id = row[0]
                                od_new_timestamp = row[1]
                                xmlFragment = row[2].encode('UTF-8')
                                # Fichier temporaire du fragment à traiter
                                xmlFragmentFile = open('../tmp/fragment.osc','wb')
                                xmlFragmentFile.write(xmlFragment)
                                xmlFragmentFile.close()
                                # Fusion du fichier final avec le fragment dans un fichier temporaire
                                with open(xmlFullFile + '.tmp','w') as fout:
                                    if platform == 'Windows':
                                        proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    else:
                                        proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    return_code = proc.wait()
                                # Le fichier temporaire devient le fichier final
                                os.remove(xmlFullFile)
                                os.remove('../tmp/fragment.osc')
                                shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                        # Si le node n'est pas trouvé dans lbt_osmdiff (car hors emprise), on génère son propre OSC
                        elif nodesRows is not None and not len(nodesRows):
                            logging.info('Node outside perimeter ' + str(node))
                            # Récupère les coordonnées du node depuis le OSC de la way
                            nodeLat = root.xpath("//nd[@ref='%s']/@lat" % node)[0]
                            nodeLon = root.xpath("//nd[@ref='%s']/@lon" % node)[0]

                            # Base de référence (table point)
                            cursor.execute(sql.SQL("SELECT osm_id FROM {0}.{1} WHERE osm_id = %s;").format(sql.Identifier(db_ref_schema),sql.Identifier(db_ref_prefix + '_point')),(node,))
                            row = cursor.fetchone()

                            # Si le node est trouvé dans la base de référence, on crée un OSC de type modify
                            if row is not None and nodeLon is not None and nodeLat is not None:
                                top = ET.Element('osmChange')
                                top.set('version', '0.6')
                                top.set('generator','LeBonTag')
                                topChild = ET.SubElement(top, 'modify')
                                topChildElement = ET.SubElement(topChild, 'node')
                                topChildElement.set('id',str(node))
                                topChildElement.set('lat',str(nodeLat))
                                topChildElement.set('lon',str(nodeLon))
                                oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                # Fichier temporaire du fragment à traiter
                                xmlFragmentFile = open('../tmp/fragment.osc','w')
                                xmlFragmentFile.write(oscContent)
                                xmlFragmentFile.close()
                                # Fusion du fichier final avec le fragment dans un fichier temporaire
                                with open(xmlFullFile + '.tmp','w') as fout:
                                    if platform == 'Windows':
                                        proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    else:
                                        proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    return_code = proc.wait()
                                # Le fichier temporaire devient le fichier final
                                os.remove(xmlFullFile)
                                os.remove('../tmp/fragment.osc')
                                shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                            # Si le node n'est pas trouvé dans la base de référence, on crée un OSC de type create
                            elif row is None and nodeLon is not None and nodeLat is not None:
                                top = ET.Element('osmChange')
                                top.set('version', '0.6')
                                top.set('generator','LeBonTag')
                                topChild = ET.SubElement(top, 'create')
                                topChildElement = ET.SubElement(topChild, 'node')
                                topChildElement.set('id',str(node))
                                topChildElement.set('lat',str(nodeLat))
                                topChildElement.set('lon',str(nodeLon))
                                oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                # Fichier temporaire du fragment à traiter
                                xmlFragmentFile = open('../tmp/fragment.osc','w')
                                xmlFragmentFile.write(oscContent)
                                xmlFragmentFile.close()
                                # Fusion du fichier final avec le fragment dans un fichier temporaire
                                with open(xmlFullFile + '.tmp','w') as fout:
                                    if platform == 'Windows':
                                        proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    else:
                                        proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                    return_code = proc.wait()
                                # Le fichier temporaire devient le fichier final
                                os.remove(xmlFullFile)
                                os.remove('../tmp/fragment.osc')
                                shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                # Pour les relations, traite chacun des membres qui peuvent être des nodes ou des ways
                if od_id_members is not None and len(od_id_members):
                    cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc,od_id_nodes FROM {0}.lbt_osmdiff WHERE od_id = ANY (%s) AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc,od_id_nodes FROM {0}.lbt_osmdiff_history WHERE od_id = ANY (%s) AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(od_id_members,od_new_timestamp,od_id_members,od_new_timestamp,))
                    membersRows = cursor.fetchall()
                    if membersRows is not None:
                        for membersRow in membersRows:
                            od_id = membersRow[0]
                            od_new_timestamp = membersRow[1]
                            xmlFragment = membersRow[2].encode('UTF-8')
                            od_id_nodes = membersRow[3]

                            # On écrit le fragment OSC de chaque membre (node ou way)
                            # Fichier temporaire du fragment à traiter
                            xmlFragmentFile = open('../tmp/fragment.osc','wb')
                            xmlFragmentFile.write(xmlFragment)
                            xmlFragmentFile.close()
                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                            with open(xmlFullFile + '.tmp','w') as fout:
                                if platform == 'Windows':
                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                else:
                                    proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                return_code = proc.wait()
                            # Le fichier temporaire devient le fichier final
                            os.remove(xmlFullFile)
                            os.remove('../tmp/fragment.osc')
                            shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                            # Récupère les nodes associés à l'objet validé (si c'est une way)
                            # Correction #58 : il faut aussi générer les OSC des nodes hors emprise de la way car ils n'ont pas été chargés dans osmdiff
                            # Donc on doit tester les nodes 1 par 1.
                            if od_id_nodes is not None and len(od_id_nodes):
                                root = ET.fromstring(xmlFragment)
                                for node in od_id_nodes:

                                    # Recherche le node dans lbt_osmdiff
                                    cursor.execute(sql.SQL("SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') UNION ALL SELECT od_id,od_new_timestamp,od_osc FROM {0}.lbt_osmdiff_history WHERE od_id = %s AND od_new_timestamp <= %s AND od_status != (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') ORDER BY od_id,od_new_timestamp;").format(sql.Identifier(lbtSchema)),(node,od_new_timestamp,node,od_new_timestamp,))
                                    nodesRows = cursor.fetchall()

                                    # Si le node est trouvé au moins une fois dans lbt_osmdiff, on inclut son propre OSC
                                    if nodesRows is not None and len(nodesRows):
                                        for row in nodesRows:
                                            logging.info('Write OSC fragment for node ' + str(row[0]))
                                            od_id = row[0]
                                            od_new_timestamp = row[1]
                                            xmlFragment = row[2].encode('UTF-8')
                                            # Fichier temporaire du fragment à traiter
                                            xmlFragmentFile = open('../tmp/fragment.osc','wb')
                                            xmlFragmentFile.write(xmlFragment)
                                            xmlFragmentFile.close()
                                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                                            with open(xmlFullFile + '.tmp','w') as fout:
                                                if platform == 'Windows':
                                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                else:
                                                    proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                return_code = proc.wait()
                                            # Le fichier temporaire devient le fichier final
                                            os.remove(xmlFullFile)
                                            os.remove('../tmp/fragment.osc')
                                            shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                                    # Si le node n'est pas trouvé dans lbt_osmdiff (car hors emprise), on génère son propre OSC
                                    elif nodesRows is not None and not len(nodesRows):
                                        logging.info('Node outside perimeter ' + str(node))
                                        # Récupère les coordonnées du node depuis le OSC de la way
                                        nodeLat = root.xpath("//nd[@ref='%s']/@lat" % node)[0]
                                        nodeLon = root.xpath("//nd[@ref='%s']/@lon" % node)[0]

                                        # Base de référence (table point)
                                        cursor.execute(sql.SQL("SELECT osm_id FROM {0}.{1} WHERE osm_id = %s;").format(sql.Identifier(db_ref_schema),sql.Identifier(db_ref_prefix + '_point')),(node,))
                                        row = cursor.fetchone()

                                        # Si le node est trouvé dans la base de référence, on crée un OSC de type modify
                                        if row is not None and nodeLon is not None and nodeLat is not None:
                                            top = ET.Element('osmChange')
                                            top.set('version', '0.6')
                                            top.set('generator','LeBonTag')
                                            topChild = ET.SubElement(top, 'modify')
                                            topChildElement = ET.SubElement(topChild, 'node')
                                            topChildElement.set('id',str(node))
                                            topChildElement.set('lat',str(nodeLat))
                                            topChildElement.set('lon',str(nodeLon))
                                            oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                            # Fichier temporaire du fragment à traiter
                                            xmlFragmentFile = open('../tmp/fragment.osc','w')
                                            xmlFragmentFile.write(oscContent)
                                            xmlFragmentFile.close()
                                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                                            with open(xmlFullFile + '.tmp','w') as fout:
                                                if platform == 'Windows':
                                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                else:
                                                    proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                return_code = proc.wait()
                                            # Le fichier temporaire devient le fichier final
                                            os.remove(xmlFullFile)
                                            os.remove('../tmp/fragment.osc')
                                            shutil.move(xmlFullFile + '.tmp',xmlFullFile)

                                        # Si le node n'est pas trouvé dans la base de référence, on crée un OSC de type create
                                        elif row is None and nodeLon is not None and nodeLat is not None:
                                            top = ET.Element('osmChange')
                                            top.set('version', '0.6')
                                            top.set('generator','LeBonTag')
                                            topChild = ET.SubElement(top, 'create')
                                            topChildElement = ET.SubElement(topChild, 'node')
                                            topChildElement.set('id',str(node))
                                            topChildElement.set('lat',str(nodeLat))
                                            topChildElement.set('lon',str(nodeLon))
                                            oscContent = ET.tostring(top, encoding='UTF-8', method="xml", pretty_print=False).decode('UTF-8')
                                            # Fichier temporaire du fragment à traiter
                                            xmlFragmentFile = open('../tmp/fragment.osc','w')
                                            xmlFragmentFile.write(oscContent)
                                            xmlFragmentFile.close()
                                            # Fusion du fichier final avec le fragment dans un fichier temporaire
                                            with open(xmlFullFile + '.tmp','w') as fout:
                                                if platform == 'Windows':
                                                    proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                else:
                                                    proc = subprocess.Popen(['osmconvert',xmlFullFile,'../tmp/fragment.osc','--out-osc','--merge-versions'],stdout=fout)
                                                return_code = proc.wait()
                                            # Le fichier temporaire devient le fichier final
                                            os.remove(xmlFullFile)
                                            os.remove('../tmp/fragment.osc')
                                            shutil.move(xmlFullFile + '.tmp',xmlFullFile)

        else:
            logging.info('No integrated or validated objects to update. Exiting.')
            sys.exit(0)

    elif os.path.isfile(xmlFile):
        with open(xmlFullFile + '.tmp','w') as fout:
            if platform == 'Windows':
                proc = subprocess.Popen([work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe',xmlFullFile,xmlFile,'--out-osc','--merge-versions'],stdout=fout)
            else:
                proc = subprocess.Popen(['osmconvert',xmlFullFile,xmlFile,'--out-osc','--merge-versions'],stdout=fout)
            return_code = proc.wait()
        # Le fichier temporaire devient le fichier final
        os.remove(xmlFullFile)
        shutil.move(xmlFullFile + '.tmp',xmlFullFile)

    else:
        logging.info('No validated objects to update. Exiting.')
        sys.exit(0)

except Exception as e:
    logging.critical('OSC fetching error : ' + str(e))
    sys.exit(1)

# Si la base de référence doit être mise à jour :
if db_ref_update == '1':

    try:
        logging.info('Change search_path for osm2pgsql.')
        cursor.execute(sql.SQL("ALTER ROLE {0} SET search_path TO {1}, public;").format(sql.Identifier(lbtUser),sql.Identifier(db_ref_schema)))
        conn.commit()
    except Exception as e:
        logging.critical('PostgreSQL error : ' + str(e))
        sys.exit(1)

    # Met à jour la BDD cible via osm2pgsql
    try:
        logging.info('Update target database via osm2pgsql.')
        if osm_stylefile is not None:
            if platform == 'Windows':
                proc = subprocess.Popen([work_directory + '/tools/osm2pgsql/osm2pgsql.exe','--append','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--style',osm_stylefile,xmlFile],env=dict(os.environ,PGPASSWORD=lbtPassword))
            else:
                proc = subprocess.Popen(['osm2pgsql','--append','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,'--style',osm_stylefile,xmlFile],env=dict(os.environ,PGPASSWORD=lbtPassword))
        else:
            if platform == 'Windows':
                proc = subprocess.Popen([work_directory + '/tools/osm2pgsql/osm2pgsql.exe','--append','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,xmlFile],env=dict(os.environ,PGPASSWORD=lbtPassword))
            else:
                proc = subprocess.Popen(['osm2pgsql','--append','--slim','--proj',str(db_ref_srid),'--hstore-all','--hstore-add-index','--multi-geometry','--keep-coastlines','--extra-attributes','--host',lbtHost,'--port',str(lbtPort),'--database',lbtDb,'--username',lbtUser,'--prefix',db_ref_prefix,xmlFile],env=dict(os.environ,PGPASSWORD=lbtPassword))
    except Exception as e:
        logging.critical('osm2pgsql execution error : ' + str(e))
        sys.exit(1)

    try:
        logging.info('Change search_path for osm2pgsql.')
        cursor.execute(sql.SQL("ALTER ROLE {0} SET search_path TO public, {1};").format(sql.Identifier(lbtUser),sql.Identifier(db_ref_schema)))
        conn.commit()
    except Exception as e:
        logging.critical('PostgreSQL error : ' + str(e))
        sys.exit(1)

    # Met à jour le statut des objets mis à jour dans la base (Validé -> Intégré)
    try:
        logging.info('Update validated objects to integrated objects.')
        cursor.execute(sql.SQL("UPDATE {0}.lbt_osmdiff SET od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'integrated') WHERE od_osc IS NOT NULL AND od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'valid');").format(sql.Identifier(lbtSchema)))
        conn.commit()
    except Exception as e:
        logging.critical('PostgreSQL error : ' + str(e))
        sys.exit(1)

# Si la base pgRouting doit être mise à jour :
if db_pgr_update == '1':

    # Applique le OSC FULL sur le fichier OSM source avec osmconvert
    logging.info('Apply OSC on OSM file')
    if os.path.isfile(osm_ref_data_file_updated):
        os.remove(osm_ref_data_file_updated)
    try:
        if platform == 'Windows':
            os.system(work_directory + '/tools/osmconvert/osmconvert64-0.8.8p.exe ' + osm_ref_data_file + ' ' + xmlFullFile + ' --out-osm -o=' + osm_ref_data_file_updated)
        else:
            os.system('osmconvert ' + osm_ref_data_file + ' ' + xmlFullFile + ' --out-osm -o=' + osm_ref_data_file_updated)
    except Exception as e:
        logging.critical('osmconvert execution error : ' + str(e))
        sys.exit(1)

    # Nettoie le fichier pour pgRouting
    try:
        logging.info('Clean OSM file')
        #with open('../osm/pbf/' + osm_pbffile + '-update-clean.osm','w') as fout:
            #proc = subprocess.Popen(['osmfilter','../osm/pbf/' + osm_pbffile + '-update.osm','--drop-version','--keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian"','--out-osm'],stdout=fout)
        if platform == 'Windows':
            os.system(work_directory + '/tools/osmfilter/osmfilter.exe ' + osm_ref_data_file_updated + ' --drop-version --keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian" --out-osm -o=' + osm_pgr_data_updated_file)
        else:
            os.system('osmfilter ' + osm_ref_data_file_updated + ' --drop-version --keep="highway=motorway =motorway_link =motorway_junction =trunk =trunk_link =primary =primary_link =secondary =secondary_link =tertiary =tertiary_link =residential =living_street =unclassified =road =track =bus_guideway =service =construction =cycleway =pedestrian =path =footway =bridleway =byway =steps railway=disused =abandoned =razed =platform tracktype =grade1 =grade2 =grade3 =grade4 =grade5 construction=motorway_link =primary =tertiary =residential =unclassified =yes =service =cycleway =pedestrian" --out-osm -o=' + osm_pgr_data_updated_file)
    except Exception as e:
        logging.critical('osmfilter execution error : ' + str(e))
        sys.exit(1)

    # Charge le fichier dans pgRouting
    try:
        logging.info('Load OSM file with osm2pgrouting')
        #proc = subprocess.run(['osm2pgrouting','--file',osm_pgr_data_updated_file,'--conf',osm_mapconfigfile,'--host',lbtHost,'--port',str(lbtPort),'--dbname',lbtDb,'--username',lbtUser,'--password',lbtPassword,'--schema',db_pgr_schema,'--prefix',db_pgr_prefix + '_','--clean','--chunk','10000000'])
        if platform == 'Windows':
            logging.info(subprocess.run([work_directory + '/tools/osm2pgrouting/osm2pgrouting.exe','--file',osm_pgr_data_updated_file,'--conf',osm_mapconfigfile,'--host',lbtHost,'--port',str(lbtPort),'--dbname',lbtDb,'--username',lbtUser,'--password',lbtPassword,'--schema',db_pgr_schema,'--prefix',db_pgr_prefix + '_','--clean','--chunk','10000000'],stdout=subprocess.PIPE,stderr=subprocess.PIPE))
        else:
            logging.info(subprocess.run(['/usr/local/bin/osm2pgrouting','--file',osm_pgr_data_updated_file,'--conf',osm_mapconfigfile,'--host',lbtHost,'--port',str(lbtPort),'--dbname',lbtDb,'--username',lbtUser,'--password',lbtPassword,'--schema',db_pgr_schema,'--prefix',db_pgr_prefix + '_','--clean','--chunk','10000000'],stdout=subprocess.PIPE,stderr=subprocess.PIPE))
    except Exception as e:
        logging.critical('osm2pgrouting execution error : ' + str(e))
        sys.exit(1)

# Ferme la connexion PG
try:
    logging.info('Close database connection.')
    cursor.close()
    conn.close()
except Exception as e:
    logging.critical('Error when closing PG : ' + str(e))
    sys.exit(1)

logging.info('Python script ended.')
